@@ -105,6 +105,8 @@ def apply_design_rules(anm):
     build_phy(anm)
     g_phy = anm['phy']
 
+    build_l3_connectivity(anm)
+
     build_vrf(anm) # need to do before to add loopbacks before ip allocations
     build_ip(anm) # ip infrastructure topology
 
@@ -215,6 +217,12 @@ def build_ibgp_vpn_v4(anm):
             n in pe_nodes and n.ibgp_role == "RRC"}
     ce_nodes = set(g_vrf.nodes(vrf_role = "CE"))
 
+    if len(pe_nodes) == len(ce_nodes) == len(pe_rrc_nodes) == 0:
+        # no vrf nodes to connect
+        return
+
+    #TODO: extend this to only connect nodes which are connected in VRFs, so don't set to others
+
     ibgp_vpn_v4_nodes = (n for n in ibgp_v4_nodes 
             if n not in pe_rrc_nodes and n not in ce_nodes)
     g_ibgp_vpn_v4.add_nodes_from(ibgp_vpn_v4_nodes, retain = "ibgp_level")
@@ -227,25 +235,6 @@ def build_ibgp_vpn_v4(anm):
     ce_edges = [e for e in g_ibgp_vpn_v4.edges()
             if e.src in ce_nodes or e.dst in ce_nodes]
 
-    """
-    #TODO: do we still need this?
-    g_ibgp_vpn_v4.remove_edges_from(ce_edges)
-
-# add CE -> PE links based on physical connectivity
-#Note: this could later look at 
-    ce_to_pe_edges = []
-    ce_phy_nodes = {g_phy.node(n) for n in ce_nodes}
-    pe_phy_nodes = {g_phy.node(n) for n in g_vrf.nodes(vrf_role = "PE")}
-    ce_to_pe_edges += (e for e in g_phy.edges()
-            if 
-            e.src.asn == e.dst.asn 
-            and
-            ((e.src in ce_phy_nodes and e.dst in pe_phy_nodes)
-            or (e.src in pe_phy_nodes and e.dst in ce_phy_nodes)))
-
-    g_ibgp_vpn_v4.add_edges_from(ce_to_pe_edges, type="ibgp", bidirectional = True)
-    """
-
     # mark ibgp direction
     ce_pe_edges = []
     pe_ce_edges = []
@@ -259,7 +248,6 @@ def build_ibgp_vpn_v4(anm):
             edge.vrf = edge.dst.vrf
             pe_ce_edges.append(edge)
 
-
     #TODO: Document this
     g_ibgpv4 = anm['ibgp_v4']
     g_ibgpv6 = anm['ibgp_v6']
@@ -282,13 +270,13 @@ def build_ibgp_vpn_v4(anm):
     g_bgp.add_edges_from(ce_pe_edges, retain = ["direction", "vrf", "type"])
     g_bgp.add_edges_from(pe_ce_edges, retain = ["direction", "vrf", "type"])
 
-
     # also need to modify the ibgp_v4 and ibgp_v6 graphs
 
 def build_mpls_ldp(anm):
     """Builds MPLS LDP"""
     g_in = anm['input']
     g_vrf = anm['vrf']
+    g_l3conn = anm['l3_conn']
     g_mpls_ldp = anm.add_overlay("mpls_ldp")
     nodes_to_add = [n for n in g_in.nodes("is_router")
             if n['vrf'].vrf_role in ("PE", "P")]
@@ -298,14 +286,19 @@ def build_mpls_ldp(anm):
     pe_nodes = set(g_vrf.nodes(vrf_role = "PE"))
     p_nodes = set(g_vrf.nodes(vrf_role = "P"))
 
-    pe_to_pe_edges = (e for e in g_in.edges()
+    pe_to_pe_edges = (e for e in g_l3conn.edges()
             if e.src in pe_nodes and e.dst in pe_nodes)
     g_mpls_ldp.add_edges_from(pe_to_pe_edges)
-    pe_to_p_edges = (e for e in g_in.edges()
+
+    pe_to_p_edges = (e for e in g_l3conn.edges()
             if e.src in pe_nodes and e.dst in p_nodes
             or e.src in p_nodes and e.dst in pe_nodes)
     g_mpls_ldp.add_edges_from(pe_to_p_edges)
 
+    p_to_p_edges = (e for e in g_l3conn.edges()
+            if e.src in p_nodes and e.dst in p_nodes)
+    g_mpls_ldp.add_edges_from(p_to_p_edges)
+
 def mark_ebgp_vrf(anm):
     g_ebgp = anm['ebgp']
     g_vrf = anm['vrf']
@@ -327,6 +320,7 @@ def mark_ebgp_vrf(anm):
 def build_vrf(anm):
     """Build VRF Overlay"""
     g_in = anm['input']
+    g_l3conn = anm['l3_conn']
     g_vrf = anm.add_overlay("vrf")
     g_vrf.add_nodes_from(g_in.nodes("is_router"), retain=["vrf_role", "vrf"])
 
@@ -339,7 +333,7 @@ def build_vrf(anm):
         dst_vrf_role = g_vrf.node(edge.dst).vrf_role
         return (src_vrf_role, dst_vrf_role) in (("PE", "CE"), ("CE", "PE"))
 
-    vrf_add_edges = (e for e in g_in.edges()
+    vrf_add_edges = (e for e in g_l3conn.edges()
            if is_pe_ce_edge(e))
     #TODO: should mark as being towards PE or CE
     g_vrf.add_edges_from(vrf_add_edges, retain=['edge_id'])
@@ -348,7 +342,7 @@ def build_vrf(anm):
         src_vrf_role = g_vrf.node(edge.src).vrf_role
         dst_vrf_role = g_vrf.node(edge.dst).vrf_role
         return (src_vrf_role, dst_vrf_role) in (("PE", "P"), ("P", "PE"))
-    vrf_add_edges = (e for e in g_in.edges()
+    vrf_add_edges = (e for e in g_l3conn.edges()
             if is_pe_p_edge(e))
     g_vrf.add_edges_from(vrf_add_edges, retain=['edge_id'])
 
@@ -862,12 +856,23 @@ def build_phy(anm):
     """Build physical overlay"""
     g_in = anm['input']
     g_phy = anm['phy']
+
+    # apply uuid if not set
+    g_phy.data.uuid = g_phy.data.uuid
+    if not g_phy.data.uuid:
+        import uuid
+        g_phy.data.uuid = uuid.uuid4().hex[:10]
+
     g_phy.add_nodes_from(g_in, retain=['label', 'update', 'device_type', 'asn',
         'specified_int_names',
         'device_subtype', 'platform', 'host', 'syntax'])
     if g_in.data.Creator == "Topology Zoo Toolset":
         ank_utils.copy_attr_from(g_in, g_phy, "Network")
 
+    g_phy.add_edges_from(g_in.edges(type="physical"))
+    # TODO: make this automatic if adding to the physical graph?
+    g_phy.allocate_interfaces() 
+
     if g_in.data.Creator == "Maestro":
         g_phy.data.mgmt_interfaces_enabled = g_in.data.mgmt_interfaces_enabled 
         g_phy.data.mgmt_address_start = g_in.data.mgmt_address_start 
@@ -876,9 +881,12 @@ def build_phy(anm):
         ank_utils.copy_attr_from(g_in, g_phy, "use_cdp")
         ank_utils.copy_attr_from(g_in, g_phy, "use_onepk")
 
-    g_phy.add_edges_from(g_in.edges(type="physical"))
-    # TODO: make this automatic if adding to the physical graph?
-    g_phy.allocate_interfaces() 
+            # copy over numeric interface ids
+        for node in g_phy:
+            for interface in node:
+                edge = interface.edges()[0]
+                directed_edge = anm['input_directed'].edge(edge)
+                interface.numeric_int_id = directed_edge.numeric_int_id
 
     for node in g_phy.nodes("specified_int_names"):
         for interface in node:
@@ -886,6 +894,26 @@ def build_phy(anm):
             directed_edge = anm['input_directed'].edge(edge)
             interface.name = directed_edge.name
 
+def build_l3_connectivity(anm):
+    """ creates l3_connectivity graph, which is switch nodes aggregated and exploded"""
+    #TODO: use this as base for ospf, ebgp, ip, etc rather than exploding in each
+    g_in = anm['input']
+    g_l3conn = anm.add_overlay("l3_conn")
+    g_l3conn.add_nodes_from(g_in, retain=['label', 'update', 'device_type', 'asn',
+        'specified_int_names',
+        'device_subtype', 'platform', 'host', 'syntax'])
+    g_l3conn.add_nodes_from(g_in.nodes("is_switch"), retain=['asn'])
+#TODO: check if edge_id needs to be still retained
+    g_l3conn.add_edges_from(g_in.edges(), retain=['edge_id'])
+
+    ank_utils.aggregate_nodes(g_l3conn, g_l3conn.nodes("is_switch"),
+                              retain="edge_id")
+    exploded_edges = ank_utils.explode_nodes(g_l3conn, g_l3conn.nodes("is_switch"),
+                            retain="edge_id")
+    for edge in exploded_edges:
+        edge.multipoint = True
+
+
 def build_conn(anm):
     """Build connectivity overlay"""
     g_in = anm['input']
